/*
 * ditoh@andrew.cmu.edu
 * MTI Spring '18
 * Assignment 1
 * 
 * Goal is to visualize the voice amplitude and the throat vibration
 * for voice training. A microphone and a piezo sensor are used as
 * inputs and used 2 LED for visualization.
 */

// pin variables
static const int micPin = A0;
static const int vibPin = A1;
static const int ledMicPin = 3;
static const int ledVibPin = 4;

// microphone variables
static const int micSize = 16;
unsigned int micReads[micSize];
unsigned int micIndex = 0;

// piezo variables
static const int vibSize = 100;
unsigned int vibReads[vibSize];
unsigned int vibIndex = 0;

// stat variables
unsigned int mean = 0;
unsigned int median = 0;
unsigned int low = 1024;
unsigned int high = 0;


void setup() {
  // serial setting
  Serial.begin(115200);

  // pin setting
  pinMode(micPin, INPUT);
  pinMode(vibPin, INPUT);
  pinMode(ledMicPin, OUTPUT);
  pinMode(ledVibPin, OUTPUT);
}


void loop() {
  // read sensor inputs every loop
  micReads[micIndex] = analogRead(micPin);
  vibReads[vibIndex] = analogRead(vibPin);

  // microphone input processing
  if (micIndex < (micSize - 1)) {
    micIndex++;
  }
  else {
    computeStat(micReads, micSize);
    // compute peak-peak amplitude
    double peakToPeak = high - low;
    peakToPeak = map(peakToPeak, 0, 1023, 0, 255);
    analogWrite(ledMicPin, peakToPeak);
    Serial.println(peakToPeak);

    // console
#if 0
    Serial.println("MICROPHONE");
    Serial.print("mean: ");
    Serial.println(mean);
    Serial.print("max: ");
    Serial.println(high);
    Serial.print("min: ");
    Serial.println(low);
    Serial.print("median: ");
    Serial.println(median);
    Serial.print("P-P amplitude: ");
    Serial.println(high - low);
    Serial.println("");
#endif

    // reset
    micIndex = 0;
  }

  // piezo input processing
  if (vibIndex < (vibSize - 1)) {
    vibIndex++;
  }
  else {
    computeStat(vibReads, vibSize);
    // scaled mean by 3 to make it sensitive
    analogWrite(ledVibPin, map(mean * 3, 0, 1023, 0, 255));
    Serial.println(mean);
    
    // console
#if 0
    Serial.println("PIEZO");
    Serial.print("mean: ");
    Serial.println(mean);
    Serial.print("max: ");
    Serial.println(high);
    Serial.print("min: ");
    Serial.println(low);
    Serial.print("median: ");
    Serial.println(median);
    Serial.println("");
#endif

    // reset
    vibIndex = 0;
  }
}


void computeStat(unsigned int readings[], unsigned int sampleSize) {
  unsigned int sum = 0;
  low = 1024;
  high = 0;

  // loop samples
  for (int i = 0; i < sampleSize; i++) {
    // ignore outliers
    if (readings[i] < 1024) {
      sum += readings[i];
      
      if (readings[i] < low) {
        low = readings[i];
      }
      if (readings[i] > high) {
        high = readings[i];
      }
    }
  }

  mean = sum / sampleSize;
  bubbleSort(readings, sampleSize);
  median = readings[sampleSize / 2];
}


// tigoe's sort for an array
void bubbleSort(unsigned int readings[], unsigned int sampleSize) {
  int out, in, swapper;
  for(out=0 ; out < sampleSize; out++) {  // outer loop
    for(in=out; in<(sampleSize-1); in++)  {  // inner loop
      if( readings[in] > readings[in+1] ) {   // out of order?
        // swap them:
        swapper = readings[in];
        readings [in] = readings[in+1];
        readings[in+1] = swapper;
      }
    }
  }
}
